#
# Todo:
#   nick:sign  <nick>                returns 0
#   nick:part  <nick>                returns 1 if 0 chans are shared (auth gets unset) and 0 if there are still chans
#


namespace eval auth {
  variable blocked 0
  variable names
  array set names {};
  bind time -|- "?0 * * * *" [namespace current]::refresh:time
  bind time -|- "?5 * * * *" [namespace current]::refresh:time
  bind nick -|- "*"          [namespace current]::nick:change
  bind raw  -|- "354"        [namespace current]::savename
  bind msg  m|- "authinfo"   [namespace current]::spewinfo
  namespace export getname savename nick:change refresh:time refresh cleanup clean search spewinfo
}

proc auth::getname {nickname} {
  variable names
  foreach username [array names names] {
    if {[string equal -nocase $nickname $username]} {
      return $names($username)
    }
  }
  return 0
}

proc auth::search {authname} {
  variable names
  set users ""
  foreach nickname [array names names] {
    if {[string equal -nocase $authname $names($nickname)]} {
      lappend users $nickname
    }
  }
  return $users
}

proc auth::savename {server rawid content} {
  variable names
  set id        [lindex [clean $content] 1]
  set nickname  [lindex [clean $content] 4]
  set authname  [lindex [clean $content] 5]
  if {($id != 666) || ($authname == 0)} {
    return 0
  }
  set names($nickname) $authname
  return $authname
}

proc auth::nick:change {nickname hostname handle channel newnickname} {
  variable names
  if {[info exists names($nickname)]} {
    set names($newnickname) $names($nickname)
    unset names($nickname)
    return $newnickname
  }
  return 0
}

proc auth::refresh:time {{args NULL}} {
  variable blocked
  if {$blocked} {
    return 0
  }
  set blocked 1
  cleanup
  refresh
  utimer 5 [list set auth::blocked 0]
  return 1
}

proc auth::refresh {{channel ""}} {
  if {$channel == ""} {
    set channels {}
    foreach channel [channels] {
      if {[botonchan $channel]} {
        lappend channels $channel
      }
    }
    set channels [join $channels {,}]
    putquick "WHO $channels c%hnuat,666"
  } else {
    putquick "WHO $channel c%hnuat,666"
  }
  return 0
}

proc auth::cleanup {} {
  variable names
  set killed 0
  foreach user [array names names] {
    set found($user) 0
    foreach channel [channels] {
      if {[onchan $user $channel]} {
        set found($user) 1
        break
      }
    }
    if {!$found($user)} {
      unset names($user)
      incr killed
    }
  }
  return $killed
}

proc auth::clean {i} {
  return [regsub -all -- {([\(\)\[\]\{\}\$\"\\])} $i {\\\1}];
}

proc auth::spewinfo {nickname hostname handle arguments} {
  variable names
  set chans [llength [channels]]
  set auths [array size names]
  foreach channel [channels] {
    if {![info exists users]} {
      set users [chanlist $channel]
      continue
    }
    append users "\x20[chanlist $channel]"
  }
  set users [llength [lsort -unique $users]]
  putquick "NOTICE $nickname :Authname information: Saved \002$auths\002 authnames of \002$users\002 users on \002$chans\002 channels."
}

putlog "Script loaded: Auth initialization by perpleXa"